 ifconst LZSSPLAYER
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; RMT tracks compressed with lzss.                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
lzss_track1:
  INCBIN "music_digdug2/BGM.lz8"
lzss_track1_end

lzss_track2:
  INCBIN "music_digdug2/HurryUp.lz8"
lzss_track2_end:

lzss_track3:
  INCBIN "music_digdug2/LevelClear.lz8"
lzss_track3_end:

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Track tables.                                                              ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
lzss_track_start_low:
  .byte $00,<lzss_track1,<lzss_track2,<lzss_track3
lzss_track_start_high:
  .byte $00,>lzss_track1,>lzss_track2,>lzss_track3

lzss_track_end_low:
  .byte $00,<lzss_track1_end,<lzss_track2_end,<lzss_track3_end
lzss_track_end_high:
  .byte $00,>lzss_track1_end,>lzss_track2_end,>lzss_track3_end

lzss_next_track:
  .byte $00,$01,$00,$00

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; LZSS player by DMSC.                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;
; LZSS Compressed SAP player for 8 match bits
; -------------------------------------------
;
; (c) 2020 DMSC
; Code under MIT license, see LICENSE file.
;
; This player uses:
;  Match length: 4 bits  (2 to 17)
;  Match offset: 4 bits  (1 to 16)
;  Min length: 2
;  Total match bits: 12 bits
;
; Compress using:
;  lzss -b 8 -o 4 input.rsap test.lz8
;
; Assemble this file with MADS assembler, the compressed song is expected in
; the `test.lz8` file at assembly time.
;
; The player needs 16 bytes of buffer for each pokey register stored, for a
; full SAP file this is 144 bytes.
;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; RAM equates.                                                               ;;
;; Occupies ~300 bytes (including lzss_request).                              ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
lzss_request      = LZSSRAM
lzss_ram_get_byte = lzss_request+1
lzss_track_index  = lzss_ram_get_byte+12
lzss_track_end    = lzss_track_index+1
lzss_frame_count  = lzss_track_end+2
lzss_temp         = lzss_frame_count+1

lzss_cur_pos      = lzss_temp+1
lzss_chn_copy     = lzss_cur_pos+1
lzss_chn_pos      = lzss_chn_copy+9
lzss_bit_data     = lzss_chn_pos+9
lzss_buffer       = lzss_bit_data+1     ; buffer is 256 bytes long

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; POKEY address.                                                             ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
LSZZ_POKEY = pokeyaddr

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Get a byte of song data. This is copied to RAM as it is self-modifying.    ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
lzss_rom_get_byte:
  lda $FFFF
  inc lzss_ram_get_byte+1
  bne lzss_skip
  inc lzss_ram_get_byte+2
lzss_skip:
  rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Initialise a track.                                                        ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
lzss_init_track:
  ldx lzss_track_index
  beq lzss_silence

  lda lzss_track_start_low,x
  sta lzss_ram_get_byte+$01
  lda lzss_track_start_high,x
  sta lzss_ram_get_byte+$02

  lda lzss_track_end_low,x    ; pointer to the end of the song
  sta lzss_track_end
  lda lzss_track_end_high,x
  sta lzss_track_end+$01

  lda #$00                    ; clear all data
  ldx #8
lzss_loop:
  sta lzss_chn_copy,x
  sta lzss_chn_pos,x
  dex
  bpl lzss_loop

  lda #$01
  sta lzss_bit_data

  lda #$09
  sta lzss_cur_pos

  ldy #0                      ; Y is current position in buffer - init to 0

  jsr lzss_ram_get_byte       ; get the first byte of song data, the channel bits
  sta lzss_temp

  ldx #9
lzss_clear:
  lsr lzss_temp
  tya
  ror                         ; A = 0 or 128
  sta lzss_chn_copy-$01,x
  jsr lzss_ram_get_byte
  sta lzss_buffer+$F0,x
  dex
  sta LSZZ_POKEY,x
  bne lzss_clear

  rts

lzss_silence:
  lda #$00
  sta LSZZ_POKEY+$01
  sta LSZZ_POKEY+$03
  sta LSZZ_POKEY+$05
  sta LSZZ_POKEY+$07
  rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Initialise.                                                                ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.lzss_init:
  ldx #$0B                    ; copy ROM routine into RAM
lzss_copy:
  lda lzss_rom_get_byte,x
  sta lzss_ram_get_byte,x
  dex
  bpl lzss_copy

  lda #$FF
  sta lzss_request
  lda #$00
  sta lzss_track_index
  sta lzss_frame_count

  jmp lzss_init_track

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Process, called every frame.                                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.lzss_process:
  lda lzss_request
  bpl lzss_new_track

  lda lzss_track_index
  beq lzss_silence

  lda paldetected             ; for PAL update every frame
  bne lzss_update_pokey

  dec lzss_frame_count        ; for NTSC drop 1 frame in every 6
  bpl lzss_update_pokey

  lda #$05
  sta lzss_frame_count

  rts

lzss_new_track:
  sta lzss_track_index
  lda #$FF
  sta lzss_request
  jmp lzss_init_track

lzss_update_pokey:
  ldy lzss_cur_pos
  ldx #8

lzss_chn_loop:                ; Loop through all "channels", one for each POKEY register
  sty lzss_cur_pos

  lda lzss_chn_copy,x         ; Get status of this stream
  bmi lzss_skip_chn           ; Negative - skip this channel
  bne lzss_copy_byte          ; If > 0 we are copying bytes

  ; We are decoding a new match/literal
  lsr lzss_bit_data           ; Get next bit
  bne lzss_got_bit
  jsr lzss_ram_get_byte       ; Not enough bits, refill!
  ror                         ; Extract a new bit and add a 1 at the high bit (from C set above)
  sta lzss_bit_data
lzss_got_bit:
  jsr lzss_ram_get_byte       ; Always read a byte, it could mean "match size/offset" or "literal byte"
  bcs lzss_store              ; Bit = 1 is "literal", bit = 0 is "match"

  pha                         ; Save A
  and #$0F
  adc #$2                     ; Adds 2 to match length (C = 0 from above)
  sta lzss_chn_copy,x         ; Store in "copy length"

  pla                         ; Restore A, get match position
  eor lzss_cur_pos
  and #$F0
  eor lzss_cur_pos            ; Add channel to position

  sta lzss_chn_pos,x          ; Store "position"

                              ; And start copying first byte
lzss_copy_byte:
  dec lzss_chn_copy,x         ; Decrease match length, increase match position
  lda lzss_chn_pos,x
  clc
  adc #$10
  sta lzss_chn_pos,x

  ; Now, read old data, jump to data store
  tay
  lda lzss_buffer,y
  ldy lzss_cur_pos

lzss_store:
  sta LSZZ_POKEY,x            ; Store to output and buffer
  sta lzss_buffer,y

lzss_skip_chn:
  dey
  dex
  bpl lzss_chn_loop           ; Next channel

  tya                         ; Increment buffer pos to channel 0 again
  clc
  adc #$19
  sta lzss_cur_pos

  lda lzss_ram_get_byte+$02
  cmp lzss_track_end+$01
  bne lzss_exit
  lda lzss_ram_get_byte+$01
  cmp lzss_track_end
  beq lzss_reinit

lzss_exit:
  rts

lzss_reinit:
  ldx lzss_track_index
  lda lzss_next_track,x
  sta lzss_track_index
  jmp lzss_init_track

 endif ; LZSSPLAYER

